"use strict";

// Ensure this module only gets included once. This is
// required for user scripts injected into all frames.
function includeOnce(userScript, initializer) {
  if (!window.userScripts) {
    window.userScripts = {};
  }
  if (!window.userScripts[userScript]) {
    window.userScripts[userScript] = true;
    if (typeof initializer === 'function') {
      initializer();
    }
    return false;
  }

  return true;
}

includeOnce("ContextMenu", function () {
  window.aloha_currentVideoElement = null;
  window.addEventListener("touchstart", function (evt) {
    var touchX = evt.changedTouches[0].pageX - window.scrollX;
    var touchY = evt.changedTouches[0].pageY - window.scrollY;

    findElementsAtPoint(touchX, touchY)
  }, true);

  // touchstart event not called for trackpad click
  // mousedown fired for touch and for trackpad
  // but mousedown not firing event for first touch
  // so we need to listen for both touchstart and mousedown events
  window.addEventListener("mousedown", function (event) {
    var touchX = event.pageX - window.scrollX;
    var touchY = event.pageY - window.scrollY;

    findElementsAtPoint(touchX, touchY)
  }, true);

  function findHrefLinkAtPoint(x, y) {
    var e = document.elementFromPoint(x, y);
    while (e && !e.href) {
      e = e.parentNode;
    }
    if (e && e.href) {
      return { href: e.href, rect: e.getBoundingClientRect() };
    }

    return null;
  }

  function findElementsAtPoint(x, y) {
    var jsonResult = {};
      
    var elements = document.elementsFromPoint(x, y);
    if (elements !== undefined) {
        for (var i = 0; i < elements.length; i++) {
          fillJsonResultForElement(elements[i], jsonResult);
        }
    }

    var hrefInfo = findHrefLinkAtPoint(x, y);
    if (hrefInfo) {
      jsonResult["hrefElement"] = elementInfo(hrefInfo.href, "", hrefInfo.rect);
    }

    jsonResult["touchPoint"] = { "x": x, "y": y };

    webkit.messageHandlers.contextMenuMessageHandler.postMessage(jsonResult);
  }

  function fillJsonResultForElement(e, jsonResult) {
    // Looking for source element inside video element
    // We need to find source element because video element doesn't have src attribute
    // The source elements sorted by order of preference so we use the first one
    var children = Array.prototype.slice.call(e.childNodes);
    for (var i = 0; i < e.childNodes.length; i++) {
      var child = e.childNodes[i];
      if (child.src != null && child.tagName == 'SOURCE') {
        e = child;
        break;
      }
    }

    while (e && !e.src) {
      e = e.parentNode;
    }

    if (e && e.src) {
      if (e.tagName == 'VIDEO' || e.parentNode.tagName == 'VIDEO') {
        var rect = e.tagName == 'VIDEO' ? e.getBoundingClientRect() : e.parentNode.getBoundingClientRect()
        if (e.src.endsWith(".mp3")) {
          jsonResult["audioElement"] = elementInfo(e.src, "", rect);
        } else {
          jsonResult["videoElement"] = elementInfo(e.src, e.title || e.parentNode.title, rect);
          aloha_currentVideoElement = jsonResult["videoElement"];
        }
        if (e.tagName == 'VIDEO') {
          aloha_currentVideoElement = e;
        } else if (e.parentNode.tagName == 'VIDEO') {
          aloha_currentVideoElement = e.parentNode;
        }
      }
      else if (e.tagName == 'AUDIO' || e.parentNode.tagName == 'AUDIO') {
        var rect = e.getBoundingClientRect();
        jsonResult["audioElement"] = elementInfo(e.src, "", rect);
      }
      else if (e.tagName == 'IMG') {
        var rect = e.getBoundingClientRect();
        jsonResult["imageElement"] = elementInfo(e.currentSrc, "", rect);
      }
    }
  }

  function elementInfo(URL, title, rect) {
    return {
      URL: URL,
      title: "",
      rect: {
        x: rect.x,
        y: rect.y,
        width: rect.width,
        height: rect.height
      }
    };
  }
});

function aloha_openCurrentVideoElementInFullscreen() {
  if (aloha_currentVideoElement !== undefined) {
    aloha_currentVideoElement.webkitEnterFullscreen()
  }
}
