"use strict";

if (webkit.messageHandlers.adBlockStatsExtension) {
  install();
}

function install() {
  function sendMessage(url, resourceType) {
    if (url) {
      webkit.messageHandlers.adBlockStatsExtension.postMessage({ url: url, resourceType: resourceType === undefined ? null : resourceType });
    }
  }

  function onLoadNativeCallback() {
    // Send back the sources of every image in the DOM back to the host application.
    [].slice.apply(document.images).forEach(function(el) {
      // If the image's natural width is zero, then it has not loaded so we
      // can assume that it may have been blocked.
      if (el.naturalWidth === 0) {
        sendMessage(el.src, "image");
      }
    });
  }

  let originalOpen = null;
  let originalSend = null;
  let originalImageSrc = null;
  let mutationObserver = null;

  function injectStatsTracking(enabled) {
    // This enable/disable section is a change from the original Focus iOS version.
    if (enabled) {
      if (originalOpen) {
        return;
      }
      window.addEventListener("load", onLoadNativeCallback, false);
    } else {
      window.removeEventListener("load", onLoadNativeCallback, false);

      if (originalOpen) { // if one is set, then all the enable code has run
        XMLHttpRequest.prototype.open = originalOpen;
        XMLHttpRequest.prototype.send = originalSend;
        Image.prototype.src = originalImageSrc;
        mutationObserver.disconnect();

        originalOpen = originalSend = originalImageSrc = mutationObserver = null;
      }
      return;
    }

    // -------------------------------------------------
    // Send ajax requests URLs to the host application
    // -------------------------------------------------
    var xhrProto = XMLHttpRequest.prototype;
    if (!originalOpen) {
      originalOpen = xhrProto.open;
      originalSend = xhrProto.send;
    }

    xhrProto.open = function(method, url) {
      this._url = url;
      return originalOpen.apply(this, arguments);
    };

    xhrProto.send = function(body) {
      // Only attach the `error` event listener once for this
      // `XMLHttpRequest` instance.
      if (!this._tpErrorHandler) {
        // If this `XMLHttpRequest` instance fails to load, we
        // can assume it has been blocked.
        this._tpErrorHandler = function() {
          sendMessage(this._url);
        };
        this.addEventListener("error", this._tpErrorHandler);
      }
      return originalSend.apply(this, arguments);
    };

    // -------------------------------------------------
    // Detect when new sources get set on Image and send them to the host application
    // -------------------------------------------------
    if (!originalImageSrc) {
      originalImageSrc = Object.getOwnPropertyDescriptor(Image.prototype, "src");
    }
    delete Image.prototype.src;
    Object.defineProperty(Image.prototype, "src", {
      get: function() {
        return originalImageSrc.get.call(this);
      },
      set: function(value) {
        // Only attach the `error` event listener once for this
        // Image instance.
        if (!this._tpErrorHandler) {
          // If this `Image` instance fails to load, we can assume
          // it has been blocked.
          this._tpErrorHandler = function() {
            sendMessage(this.src, "image");
          };
          this.addEventListener("error", this._tpErrorHandler);
        }

        originalImageSrc.set.call(this, value);
      }
    });
  }

  injectStatsTracking(true);
}
