function prepareAllElements(e) {
    if (document.location.hostname.includes('msn.com')) { return }
    prepareVideoElementInDocument(document);

    var iframes = document.getElementsByTagName('iframe');

    for (var i = 0; i < iframes.length; i++) {
        iframe = iframes[i];
        if (iframe.src.match(document.domain)) {
            var innerDoc = iframe.contentDocument || iframe.contentWindow.document;

            prepareVideoElementInDocument(innerDoc);
        }
    }
}

function prepareVideoElementInDocument(document) {
    var arrayOfVideoElements = document.getElementsByTagName('video');

    for (var i = 0; i < arrayOfVideoElements.length; i++) {
        var videoElement = arrayOfVideoElements[i];
        prepareVideoElement(videoElement);
    }
}

function prepareVideoElement(videoElement) {
    videoElement.removeEventListener('webkitbeginfullscreen', onFullScreen);
    videoElement.addEventListener('webkitbeginfullscreen', onFullScreen);
}

function onFullScreen(event) {
    var videoElement = event.target;
    handleEnterFullscreen(videoElement);
}

function onExitFullScreen(event) {
    var videoElement = event.target;
    handleExitFullscreen(videoElement);
}

function handleEnterFullscreen(videoElement) {
    if (videoElement.parentElement == null) {
        return;
    }

    var src = getVideoSrc(videoElement);

    var interval = setInterval(function () {
        var newSrc = getVideoSrc(videoElement);
        if (videoElement.webkitDisplayingFullscreen) {
            if (newSrc !== src) {
                postSrcMessage(videoElement, newSrc);
                src = newSrc;
            }
        } else {
            clearInterval(interval);
        }
    }, 200);

    postSrcMessage(videoElement, src);
}

function getVideoSrc(videoElement) {
    var src = videoElement.src;

    if (!src) {
        var sources = videoElement.getElementsByTagName('source');
        if (sources && sources.length > 0) {
            // Try to find any source which is not pointing to .webm video
            for (var i = 0; i < sources.length; i++) {
                if (!sources[i].src.endsWith(".webm")) {
                    src = sources[i].src;
                    break;
                }
            }

            // Fallback to .webm source and try to load it
            if (!src) {
                src = sources[0].src;
            }
        }
    }

    return src;
}

function postSrcMessage(videoElement, src) {
    var jsonResult = {
        'action': 'srcChange',
        'src': src,
        'title': videoElement.title
    };

    webkit.messageHandlers.videoPlayHandler.postMessage(jsonResult);
}

var observer = new MutationObserver(function (mutations) {
    mutations.forEach(function (mutation) {
        if (mutation.type == 'childList' && mutation.addedNodes.length) {
            for (var i = 0; i < mutation.addedNodes.length; i++) {

                node = mutation.addedNodes[i];

                if (node.tagName == 'VIDEO') {
                    prepareVideoElement(node);
                }

                if (node.getElementsByTagName) {
                    var arrayOfVideoElements = node.getElementsByTagName('video');

                    for (var j = 0; j < arrayOfVideoElements.length; j++) {
                        var videoElement = arrayOfVideoElements[j];
                        prepareVideoElement(videoElement);
                    }
                }
            }
        }
    });
});

try {
    prepareAllElements(document);
} catch (err) { }

try {
    var observerConfig = {
        attributes: false,
        childList: true,
        characterData: false,
        subtree: true,
        attributeOldValue: false,
        characterDataOldValue: false
    };
    observer.observe(document.getElementsByTagName('body')[0], observerConfig);
} catch (err) { }
